"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const formatDuration_1 = require("./formatDuration");
const taskId_1 = require("../task/taskId");
const index_1 = require("./index");
const _1 = require(".");
const chalk_1 = __importDefault(require("chalk"));
function hr() {
    _1.logger.info("----------------------------------------------");
}
function reportSummary(context) {
    const { measures } = context;
    const taskLogs = index_1.getTaskLogs();
    const statusColorFn = {
        success: chalk_1.default.greenBright,
        failed: chalk_1.default.redBright,
        skipped: chalk_1.default.gray,
    };
    hr();
    _1.logger.info(chalk_1.default.cyanBright(`🏗 Summary\n`));
    if (measures.failedTask) {
        const { pkg, task } = measures.failedTask;
        const taskId = taskId_1.getTaskId(pkg, task);
        _1.logger.error(`ERROR DETECTED IN ${pkg} ${task}`);
        _1.logger.error(taskLogs.get(taskId).join("\n"));
        hr();
    }
    if (measures.taskStats.length > 0) {
        for (const stats of measures.taskStats) {
            const colorFn = statusColorFn[stats.status];
            _1.logger.info(index_1.getTaskLogPrefix(stats.pkg, stats.task), colorFn(`${stats.status}, took ${formatDuration_1.formatDuration(stats.duration)}`));
        }
    }
    else {
        _1.logger.info("Nothing has been run.");
    }
    hr();
    _1.logger.info(`Took a total of ${formatDuration_1.formatDuration(measures.duration)} to complete`);
}
exports.reportSummary = reportSummary;
