"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const abortSignal_1 = require("./abortSignal");
const findNpmClient_1 = require("../workspace/findNpmClient");
const child_process_1 = require("child_process");
const logger_1 = require("../logger");
const taskWrapper_1 = require("./taskWrapper");
const path_1 = __importDefault(require("path"));
const p_queue_1 = __importDefault(require("p-queue"));
function wait(time) {
    return new Promise((resolve, reject) => {
        setTimeout(resolve, time);
    });
}
let npmCmd;
let queue;
function npmTask(task, info, config, context, root) {
    const { node, args, npmClient, concurrency } = config;
    const logger = logger_1.taskLogger(info.name, task);
    // cached npmCmd
    queue = queue || new p_queue_1.default({ concurrency });
    npmCmd = npmCmd || findNpmClient_1.findNpmClient(npmClient);
    const npmArgs = [...node, "run", task, "--", ...args];
    return queue.add(() => taskWrapper_1.taskWrapper(info, task, () => new Promise((resolve, reject) => {
        if (!info.scripts || !info.scripts[task]) {
            logger.info(`Empty script detected, skipping`);
            return resolve();
        }
        logger.verbose(`Running ${[npmCmd, ...npmArgs].join(" ")}`);
        const cp = child_process_1.spawn(npmCmd, npmArgs, {
            cwd: path_1.default.dirname(info.packageJsonPath),
            stdio: "pipe",
        });
        abortSignal_1.signal.addEventListener("abort", terminate);
        const stdoutLogger = new logger_1.NpmLogWritable(info.name, task);
        cp.stdout.pipe(stdoutLogger);
        const stderrLogger = new logger_1.NpmLogWritable(info.name, task);
        cp.stderr.pipe(stderrLogger);
        cp.on("exit", (code) => {
            abortSignal_1.signal.removeEventListener("abort", terminate);
            if (code === 0) {
                return resolve();
            }
            abortSignal_1.controller.abort();
            reject();
        });
        function terminate() {
            queue.pause();
            queue.clear();
            cp.kill("SIGKILL");
        }
    }).then(() => wait(100)), config, context, root));
}
exports.npmTask = npmTask;
