"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const backfill_1 = require("../cache/backfill");
const formatDuration_1 = require("../logger/formatDuration");
const logger_1 = require("../logger");
async function taskWrapper(info, task, fn, config, context, root) {
    const { profiler, measures } = context;
    const pkg = info.name;
    const logger = logger_1.taskLogger(pkg, task);
    const start = process.hrtime();
    let cacheHit = true;
    let hash = null;
    if (config.cache) {
        hash = await backfill_1.cacheHash(task, info, root, config);
        if (hash) {
            cacheHit = await backfill_1.cacheFetch(hash, info, config);
        }
        logger.verbose(`hash: ${hash}, cache hit? ${cacheHit}`);
    }
    if (!cacheHit) {
        logger.info("▶️ start");
        try {
            await profiler.run(() => fn(), `${pkg}.${task}`);
            const duration = process.hrtime(start);
            measures.taskStats.push({
                pkg,
                task,
                start,
                duration,
                status: "success",
            });
            logger.info(`✔️ done - took ${formatDuration_1.formatDuration(duration)}`);
            if (config.cache && hash) {
                logger.verbose(`hash put ${hash}`);
                await backfill_1.cachePut(hash, info, config);
            }
        }
        catch (e) {
            logger.info("❌ fail");
            if (!measures.failedTask) {
                measures.failedTask = { pkg, task };
            }
            const duration = process.hrtime(start);
            measures.taskStats.push({ pkg, task, start, duration, status: "failed" });
            throw e;
        }
    }
    else {
        const duration = process.hrtime(start);
        measures.taskStats.push({ pkg, task, start, duration, status: "skipped" });
        logger.info("⏭️ skip");
    }
}
exports.taskWrapper = taskWrapper;
